==> react-native-audio-device-1.1.3.tgz: package/android/src/main/java/com/audio/device/manager/VoiceRecorder.java <==
package com.audio.device.manager;

import android.content.Context;
import android.media.AudioDeviceInfo;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioRecord;
import android.media.AudioTrack;
import android.media.MediaRecorder;
import android.util.Log;
import androidx.annotation.NonNull;
import com.facebook.react.bridge.ReactApplicationContext;

public class VoiceRecorder {
  private static final int[] SAMPLE_RATE_CANDIDATES = new int[] {
          16000,
          11025,
          22050,
          44100,
  };

  private static final int CHANNEL = AudioFormat.CHANNEL_IN_MONO;
  private static final int ENCODING = AudioFormat.ENCODING_PCM_16BIT;

  private static final int AMPLITUDE_THRESHOLD = 1500;
  private static final int SPEECH_TIMEOUT_MILLIS = 2000;
  private static final int MAX_SPEECH_LENGTH_MILLIS = 30 * 1000;

  public abstract static class Callback {

    /**
     * Called when the recorder starts hearing voice.
     */
    public void onVoiceStart() {}

    /**
     * Called when the recorder is hearing voice.
     *
     * @param data The audio data in {@link AudioFormat#ENCODING_PCM_16BIT}.
     * @param size The size of the actual data in {@code data}.
     */
    public void onVoice(byte[] data, int size) {}

    /**
     * Called when the recorder stops hearing voice.
     */
    public void onVoiceEnd() {}
  }

  private static final int SAMPLING_RATE_IN_HZ = getMinSupportedSampleRate();

  private final Callback mCallback;

  private AudioRecord mAudioRecord;

  private Thread mThread;

  private byte[] mBuffer;

  private final Object mLock = new Object();

  /** The timestamp of the last time that voice is heard. */
  private long mLastVoiceHeardMillis = Long.MAX_VALUE;

  /** The timestamp when the current voice is started. */
  private long mVoiceStartedMillis;
  private final ReactApplicationContext reactContext;

  public VoiceRecorder(
          @NonNull Callback callback,
          ReactApplicationContext reactContext
  ) {
    mCallback = callback;
    this.reactContext = reactContext;
  }

  /**
   * Starts recording audio.
   *
   * <p>The caller is responsible for calling {@link #stop()} later.</p>
   */
  public void start() {
    // Stop recording if it is currently ongoing.
    stop();
    // Try to create a new recording session.
    mAudioRecord = createAudioRecord();
    if (mAudioRecord == null) {
      throw new RuntimeException("Cannot instantiate VoiceRecorder");
    }

    // Start recording.
    mAudioRecord.startRecording();
    // Start processing the captured audio.
    mThread = new Thread(new ProcessVoice());
    mThread.start();
  }

  /**
   * Stops recording audio.
   */
  public void stop() {
    synchronized (mLock) {
      dismiss();
      if (mThread != null) {
        mThread.interrupt();
        mThread = null;
      }
      if (mAudioRecord != null) {
        mAudioRecord.stop();
        mAudioRecord.release();
        mAudioRecord = null;
      }
      mBuffer = null;
    }
  }

  /**
   * Dismisses the currently ongoing utterance.
   */
  public void dismiss() {
    if (mLastVoiceHeardMillis != Long.MAX_VALUE) {
      mLastVoiceHeardMillis = Long.MAX_VALUE;
      mCallback.onVoiceEnd();
    }
  }

  /**
   * Retrieves the sample rate currently used to record audio.
   *
   * @return The sample rate of recorded audio.
   */
  public int getSampleRate() {
    if (mAudioRecord != null) {
      return mAudioRecord.getSampleRate();
    }
    return 0;
  }

  private static int getMinSupportedSampleRate() {
    final int[] validSampleRates = new int[] {
            8000,
            11025,
            16000,
            22050,
            32000,
            37800,
            44056,
            44100,
            47250,
            48000,
            50000,
            50400,
            88200,
            96000,
            176400,
            192000,
            352800,
            2822400,
            5644800,
    };
    /*
     * Selecting default audio input source for recording since
     * AudioFormat.CHANNEL_CONFIGURATION_DEFAULT is deprecated and selecting
     * default encoding format.
     */
    for (int validSampleRate : validSampleRates) {
      int result = AudioRecord.getMinBufferSize(
              validSampleRate,
              CHANNEL,
              ENCODING
      );
      if (result > 0) {
        // return the minimum supported audio sample rate
        return 48000;
      }
    }
    // If none of the sample rates are supported return -1 and handle it in calling method
    return -1;
  }

  /**
   * Creates a new {@link AudioRecord}.
   *
   * @return A newly created {@link AudioRecord}, or null if it cannot be created (missing
   * permissions?).
   */
  private AudioRecord createAudioRecord() {
    final int sizeInBytes = AudioRecord.getMinBufferSize(
            SAMPLING_RATE_IN_HZ,
            CHANNEL,
            ENCODING
    );

    final AudioRecord audioRecord = new AudioRecord(
            MediaRecorder.AudioSource.MIC,
            SAMPLING_RATE_IN_HZ,
            CHANNEL,
            ENCODING,
            sizeInBytes
    );
    if (audioRecord.getState() == AudioRecord.STATE_INITIALIZED) {
      mBuffer = new byte[sizeInBytes];
      return audioRecord;
    } else {
      audioRecord.release();
      return null;
    }
  }

  /**
   * Continuously processes the captured audio and notifies {@link #mCallback} of corresponding
   * events.
   */
  private class ProcessVoice implements Runnable {

    @Override
    public void run() {
      final int sizeInBytes = AudioRecord.getMinBufferSize(
              SAMPLING_RATE_IN_HZ,
              CHANNEL,
              ENCODING
      );

      AudioTrack audio = new AudioTrack(
              AudioManager.STREAM_VOICE_CALL,
              SAMPLING_RATE_IN_HZ,
              AudioFormat.CHANNEL_OUT_MONO,
              AudioFormat.ENCODING_PCM_16BIT,
              sizeInBytes,
              AudioTrack.MODE_STREAM
      );
      setPreferredOutputDevice(audio);
      audio.play();

      while (true) {
        synchronized (mLock) {
          if (Thread.currentThread().isInterrupted()) {
            break;
          }

          final int size = mAudioRecord.read(mBuffer, 0, mBuffer.length);

          if (size > 0) {
            audio.write(mBuffer, 0, size);
          }

          final long now = System.currentTimeMillis();
          if (isHearingVoice(mBuffer, size)) {
            if (mLastVoiceHeardMillis == Long.MAX_VALUE) {
              mVoiceStartedMillis = now;
              mCallback.onVoiceStart();
            }

            mCallback.onVoice(mBuffer, size);
            mLastVoiceHeardMillis = now;
            if (now - mVoiceStartedMillis > MAX_SPEECH_LENGTH_MILLIS) {
              end();
            }
          } else if (mLastVoiceHeardMillis != Long.MAX_VALUE) {
            mCallback.onVoice(mBuffer, size);
            if (now - mLastVoiceHeardMillis > SPEECH_TIMEOUT_MILLIS) {
              end();
            }
          }
        }
      }
    }

    private void end() {
      mLastVoiceHeardMillis = Long.MAX_VALUE;
      mCallback.onVoiceEnd();
    }

    private boolean isHearingVoice(byte[] buffer, int size) {
      for (int i = 0; i < size - 1; i += 2) {
        // The buffer has LINEAR16 in little endian.
        int s = buffer[i + 1];
        if (s < 0) s *= -1;
        s <<= 8;
        s += Math.abs(buffer[i]);
        if (s > AMPLITUDE_THRESHOLD) {
          return true;
        }
      }
      return false;
    }
  }

  /**
   * Function to set the preferred input device to a connected AUX line (preferably) or
   * the built-in speakers if the AUX line is not connected.
   * Function has no effect on Android version below Android 6.0 Marshmallow
   *
   * @param audio: AudioTrack instance
   */
  private void setPreferredOutputDevice(AudioTrack audio) {
    if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.M) {
      AudioManager audioManager = (AudioManager) this.reactContext.getSystemService(
              Context.AUDIO_SERVICE
      );
      AudioDeviceInfo[] outputs = audioManager.getDevices(
              AudioManager.GET_DEVICES_OUTPUTS
      );
      for (AudioDeviceInfo output : outputs) {
        if (
                output.getType() == AudioDeviceInfo.TYPE_AUX_LINE ||
                        output.getType() == AudioDeviceInfo.TYPE_WIRED_HEADPHONES ||
                        output.getType() == AudioDeviceInfo.TYPE_WIRED_HEADSET
        ) {
          audio.setPreferredDevice(output);
          break;
        } else if (output.getType() == AudioDeviceInfo.TYPE_BUILTIN_SPEAKER) {
          audio.setPreferredDevice(output);
        }
      }
    }
  }
}

==> react-native-audio-device-1.1.6.tgz: package/android/src/main/java/com/audio/device/manager/VoiceRecorder.java <==
package com.audio.device.manager;

import android.content.Context;
import android.media.AudioDeviceInfo;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioRecord;
import android.media.AudioTrack;
import android.media.MediaRecorder;
import android.util.Log;
import androidx.annotation.NonNull;
import com.facebook.react.bridge.ReactApplicationContext;

public class VoiceRecorder {
  private static final int[] SAMPLE_RATE_CANDIDATES = new int[]{16000, 11025, 22050, 44100};
  private static final int CHANNEL = AudioFormat.CHANNEL_IN_MONO;
  private static final int ENCODING = AudioFormat.ENCODING_PCM_16BIT;

  private static final int AMPLITUDE_THRESHOLD = 1500;
  private static final int SPEECH_TIMEOUT_MILLIS = 2000;
  private static final int MAX_SPEECH_LENGTH_MILLIS = 30 * 1000;

  public abstract static class Callback {

    /**
     * Called when the recorder starts hearing voice.
     */
    public void onVoiceStart() {}

    /**
     * Called when the recorder is hearing voice.
     *
     * @param data The audio data in {@link AudioFormat#ENCODING_PCM_16BIT}.
     * @param size The size of the actual data in {@code data}.
     */
    public void onVoice(byte[] data, int size) {}

    /**
     * Called when the recorder stops hearing voice.
     */
    public void onVoiceEnd() {}
  }

  private final Callback mCallback;

  private AudioRecord mAudioRecord;

  private Thread mThread;

  private byte[] mBuffer;

  private final Object mLock = new Object();

  /** The timestamp of the last time that voice is heard. */
  private long mLastVoiceHeardMillis = Long.MAX_VALUE;

  /** The timestamp when the current voice is started. */
  private long mVoiceStartedMillis;
  private final ReactApplicationContext reactContext;

  public VoiceRecorder(
          @NonNull Callback callback,
          ReactApplicationContext reactContext
  ) {
    mCallback = callback;
    this.reactContext = reactContext;
  }

  /**
   * Starts recording audio.
   *
   * <p>The caller is responsible for calling {@link #stop()} later.</p>
   */
  public void start() {
    // Stop recording if it is currently ongoing.
    stop();
    // Try to create a new recording session.
    mAudioRecord = createAudioRecord();
    if (mAudioRecord == null) {
      throw new RuntimeException("Cannot instantiate VoiceRecorder");
    }

    // Start recording.
    mAudioRecord.startRecording();
    // Start processing the captured audio.
    mThread = new Thread(new ProcessVoice());
    mThread.start();
  }

  /**
   * Stops recording audio.
   */
  public void stop() {
    synchronized (mLock) {
      dismiss();
      if (mThread != null) {
        mThread.interrupt();
        mThread = null;
      }
      if (mAudioRecord != null) {
        mAudioRecord.stop();
        mAudioRecord.release();
        mAudioRecord = null;
      }
      mBuffer = null;
    }
  }

  /**
   * Dismisses the currently ongoing utterance.
   */
  public void dismiss() {
    if (mLastVoiceHeardMillis != Long.MAX_VALUE) {
      mLastVoiceHeardMillis = Long.MAX_VALUE;
      mCallback.onVoiceEnd();
    }
  }

  /**
   * Retrieves the sample rate currently used to record audio.
   *
   * @return The sample rate of recorded audio.
   */
  public int getSampleRate() {
    if (mAudioRecord != null) {
      return mAudioRecord.getSampleRate();
    }
    return 0;
  }

  /**
   * Creates a new {@link AudioRecord}.
   *
   * @return A newly created {@link AudioRecord}, or null if it cannot be created (missing
   * permissions?).
   */
  private AudioRecord createAudioRecord() {
    for (int sampleRate : SAMPLE_RATE_CANDIDATES) {
      final int sizeInBytes = AudioRecord.getMinBufferSize(sampleRate, CHANNEL, ENCODING);
      if (sizeInBytes == AudioRecord.ERROR_BAD_VALUE) {
        continue;
      }
      final AudioRecord audioRecord = new AudioRecord(MediaRecorder.AudioSource.MIC,
              sampleRate, CHANNEL, ENCODING, sizeInBytes);
      if (audioRecord.getState() == AudioRecord.STATE_INITIALIZED) {
        mBuffer = new byte[sizeInBytes];
        return audioRecord;
      } else {
        audioRecord.release();
      }
    }
    return null;
  }

  /**
   * Continuously processes the captured audio and notifies {@link #mCallback} of corresponding
   * events.
   */
  private class ProcessVoice implements Runnable {

    @Override
    public void run() {
      final int sampleRate = getSampleRate();
      final int sizeInBytes = AudioRecord.getMinBufferSize(
              sampleRate,
              CHANNEL,
              ENCODING
      );

      AudioTrack audio = new AudioTrack(
              AudioManager.STREAM_VOICE_CALL,
              sampleRate,
              AudioFormat.CHANNEL_OUT_MONO,
              AudioFormat.ENCODING_PCM_16BIT,
              sizeInBytes,
              AudioTrack.MODE_STREAM
      );
      setPreferredOutputDevice(audio);
//      audio.play();

      while (true) {
        synchronized (mLock) {
          if (Thread.currentThread().isInterrupted()) {
            break;
          }

          final int size = mAudioRecord.read(mBuffer, 0, mBuffer.length);

          if (size > 0) {
//            audio.write(mBuffer, 0, size);
          }

          final long now = System.currentTimeMillis();
          if (isHearingVoice(mBuffer, size)) {
            if (mLastVoiceHeardMillis == Long.MAX_VALUE) {
              mVoiceStartedMillis = now;
              mCallback.onVoiceStart();
            }

            mCallback.onVoice(mBuffer, size);
            mLastVoiceHeardMillis = now;
            if (now - mVoiceStartedMillis > MAX_SPEECH_LENGTH_MILLIS) {
              end();
            }
          } else if (mLastVoiceHeardMillis != Long.MAX_VALUE) {
            mCallback.onVoice(mBuffer, size);
            if (now - mLastVoiceHeardMillis > SPEECH_TIMEOUT_MILLIS) {
              end();
            }
          }
        }
      }
    }

    private void end() {
      mLastVoiceHeardMillis = Long.MAX_VALUE;
      mCallback.onVoiceEnd();
    }

    private boolean isHearingVoice(byte[] buffer, int size) {
      for (int i = 0; i < size - 1; i += 2) {
        // The buffer has LINEAR16 in little endian.
        int s = buffer[i + 1];
        if (s < 0) s *= -1;
        s <<= 8;
        s += Math.abs(buffer[i]);
        if (s > AMPLITUDE_THRESHOLD) {
          return true;
        }
      }
      return false;
    }
  }

  /**
   * Function to set the preferred input device to a connected AUX line (preferably) or
   * the built-in speakers if the AUX line is not connected.
   * Function has no effect on Android version below Android 6.0 Marshmallow
   *
   * @param audio: AudioTrack instance
   */
  private void setPreferredOutputDevice(AudioTrack audio) {
    if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.M) {
      AudioManager audioManager = (AudioManager) this.reactContext.getSystemService(
              Context.AUDIO_SERVICE
      );
      AudioDeviceInfo[] outputs = audioManager.getDevices(
              AudioManager.GET_DEVICES_OUTPUTS
      );
      for (AudioDeviceInfo output : outputs) {
        if (
                output.getType() == AudioDeviceInfo.TYPE_AUX_LINE ||
                        output.getType() == AudioDeviceInfo.TYPE_WIRED_HEADPHONES ||
                        output.getType() == AudioDeviceInfo.TYPE_WIRED_HEADSET
        ) {
          audio.setPreferredDevice(output);
          break;
        } else if (output.getType() == AudioDeviceInfo.TYPE_BUILTIN_SPEAKER) {
          audio.setPreferredDevice(output);
        }
      }
    }
  }
}
